@@ -5,10 +5,10 @@ from dataclasses import dataclass
 
 import numpy as np
 from scipy import interpolate
-from PyQt5.QtGui import (QBrush, QPen, QColor, QPalette, QPainter, QPainterPath,
+from PyQt6.QtGui import (QBrush, QPen, QColor, QPalette, QPainter, QPainterPath,
     QCursor)
-from PyQt5.QtWidgets import QFrame
-from PyQt5.QtCore import Qt, pyqtSignal, QTimer, QPointF, QRectF, QRect
+from PyQt6.QtWidgets import QFrame
+from PyQt6.QtCore import Qt, pyqtSignal, QTimer, QPointF, QRectF, QRect
 from slider.beatmap import Circle, Slider, Spinner
 from circleguard import (Mod, Key, hitradius, hitwindows, JudgmentType,
     KeylessCircleguard)
@@ -137,27 +137,8 @@ class Renderer(QFrame):
                 easy=self.use_ez)
             self.playback_end = self.get_hit_endtime(self.hit_objects[-1])
 
-            ar = beatmap.ar(hard_rock=self.use_hr, easy=self.use_ez)
-            od = beatmap.od(hard_rock=self.use_hr, easy=self.use_ez)
-            cs = beatmap.cs(hard_rock=self.use_hr, easy=self.use_ez)
-            # see https://osu.ppy.sh/help/wiki/Beatmapping/Approach_rate
-            if ar <= 5:
-                self.preempt = 1200 + 600 * (5 - ar) / 5
-                self.fade_in = 800 + 400 * (5 - ar) / 5
-            else:
-                self.preempt = 1200 - 750 * (ar - 5) / 5
-                self.fade_in = 800 - 500 * (ar - 5) / 5
+            self.calculate_beatmap_stats(self.use_hr, self.use_ez)
 
-            (hitwindow_50, hitwindow_100, hitwindow_300) = hitwindows(od)
-            self.hitwindow_50 = hitwindow_50
-            self.hitwindow_100 = hitwindow_100
-            self.hitwindow_300 = hitwindow_300
-
-            # how much to scale our error bar by from its 'standard' size, where
-            # each ms of error is a pixel.
-            self.error_bar_width_factor = ERROR_BAR_WIDTH / (hitwindow_50 * 2)
-
-            self.hitcircle_radius = hitradius(cs)
             # loading stuff
             self.is_loading = True
             self.num_hitobjects = len(self.hit_objects)
@@ -213,11 +194,15 @@ class Renderer(QFrame):
         self.timer = QTimer(self)
         self.timer.timeout.connect(self.next_frame_from_timer)
         # 62 fps (1000ms / 60frames but the result can only be a integer)
-        self.timer.start(1000/60)
+        self.timer.start(int(1000/60))
 
         # black background
         pal = QPalette()
-        pal.setColor(QPalette.Background, Qt.black)
+        pal.setColor(QPalette.ColorGroup.Normal,
+            QPalette.ColorRole.Window, Qt.GlobalColor.black)
+        # also set when app is in background
+        pal.setColor(QPalette.ColorGroup.Inactive,
+            QPalette.ColorRole.Window, Qt.GlobalColor.black)
         self.setAutoFillBackground(True)
         self.setPalette(pal)
 
@@ -284,7 +269,7 @@ class Renderer(QFrame):
         return QPointF(self._x(x), self._y(y))
 
     def scaled_number(self, n):
-        return n * self.scale
+        return int(n * self.scale)
 
     def next_frame_from_timer(self):
         """
@@ -326,7 +311,6 @@ class Renderer(QFrame):
             self.previously_loading = True
             self.update()
             return
-
         current_time = self.clock.get_time()
         # if we're at the end of the track or are at the beginning of the track
         # (and thus are reversing), pause and dont update
@@ -351,9 +335,13 @@ class Renderer(QFrame):
             if player.end_pos >= self.num_frames_on_screen:
                 player.start_pos = player.end_pos - self.num_frames_on_screen
 
+            # never go out of bounds
+            if player.end_pos >= len(player.xy):
+                player.end_pos = len(player.xy) - 1
+
         if self.has_beatmap:
             self.get_hitobjects()
-        self.update_time_signal.emit(current_time)
+        self.update_time_signal.emit(int(current_time))
         self.update()
 
     def get_hitobjects(self):
@@ -392,8 +380,8 @@ class Renderer(QFrame):
         Called whenever self.update() is called
         """
         self.painter.begin(self)
-        self.painter.setRenderHint(QPainter.TextAntialiasing, True)
-        self.painter.setRenderHint(QPainter.Antialiasing, True)
+        self.painter.setRenderHint(QPainter.RenderHint.TextAntialiasing, True)
+        self.painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
         self.painter.setPen(PEN_WHITE)
         _pen = self.painter.pen()
         # loading screen
@@ -447,8 +435,13 @@ class Renderer(QFrame):
         self.painter.setPen(pen)
         highlighted_pen = False
         for i in range(player.start_pos, player.end_pos):
-            highlight = any((player.t[i + 1] in self.events, player.t[i] in
-                self.events))
+            # don't draw a cursor at the very last frame, this is already taken
+            # care of by the `i - 1` iteration, since we do `i + 1` inside this
+            # loop
+            if i == len(player.xy) - 1:
+                continue
+            highlight = (player.t[i] in self.events or player.t[i + 1] in
+                self.events)
             if highlight and not highlighted_pen:
                 self.painter.setPen(PEN_HIGHLIGHT)
                 highlighted_pen = True
@@ -470,7 +463,11 @@ class Renderer(QFrame):
                     player.xy[i + 1], grey_out=grey_out)
         pen.setWidth(self.scaled_number(WIDTH_CROSS))
         self.painter.setPen(pen)
-        for i in range(player.start_pos, player.end_pos+1):
+        for i in range(player.start_pos, player.end_pos + 1):
+            # avoid out of bounds error on the last frame, might be unecessary
+            # but would prefer caution
+            if i >= len(player.xy):
+                continue
             alpha = (i - player.start_pos) * alpha_step
             xy = player.xy[i]
             k = player.k[i]
@@ -597,16 +594,20 @@ class Renderer(QFrame):
                 self.painter.setPen(PEN_BLANK)
                 self.painter.setBrush(QBrush(pen.color()))
                 keys = Key(int(player.k[player.end_pos]))
-                _set_opacity(1 if Key.M1 in keys else 0.3)
+                _set_opacity(1 if Key.M1 in keys and Key.K1 not in keys else 0.3)
                 self.painter.drawRect(5, y - 9, 10, 10)
-                _set_opacity(1 if Key.M2 in keys else 0.3)
+                _set_opacity(1 if Key.M2 in keys and Key.K2 not in keys else 0.3)
                 self.painter.drawRect(18, y - 9, 10, 10)
+                _set_opacity(1 if Key.K1 | Key.M1 in keys else 0.3)
+                self.painter.drawRect(31, y - 9, 10, 10)
+                _set_opacity(1 if Key.K2 | Key.M2 in keys else 0.3)
+                self.painter.drawRect(44, y - 9, 10, 10)
                 _set_opacity(1)
                 self.painter.setPen(pen)
                 info_text = (f"{player.username} {player.mods.short_name()}: "
                     f"{player.xy[player.end_pos][0]:.2f}, "
                     f"{player.xy[player.end_pos][1]:.2f}")
-                self.painter.drawText(31, y, info_text)
+                self.painter.drawText(57, y, info_text)
                 # not sure why we need to do ``y - 9`` instead of 9 here,
                 # our ``drawText`` call is perfectly happy to accept ``y`` but
                 # we need to pass ``y - 9`` to our ``drawRect`` calls...maybe 9
@@ -620,9 +621,9 @@ class Renderer(QFrame):
                 # adjustment; would like to figure out why but this works for
                 # now.
                 info_pos.height -= 3
-                # our bounding rect starts at 5 but the text starts at 31, so
+                # our bounding rect starts at 5 but the text starts at 57, so
                 # we need to increase the width by the difference to account
-                info_pos.width += 31 - 5
+                info_pos.width += 57 - 5
                 self.player_info_positions[info_pos] = player
 
             self.painter.setOpacity(1)
@@ -845,8 +846,8 @@ class Renderer(QFrame):
         p = hitobj.position
 
         PEN_GRAY.setWidth(self.scaled_number(self.hitcircle_radius * 2))
-        PEN_GRAY.setCapStyle(Qt.RoundCap)
-        PEN_GRAY.setJoinStyle(Qt.RoundJoin)
+        PEN_GRAY.setCapStyle(Qt.PenCapStyle.RoundCap)
+        PEN_GRAY.setJoinStyle(Qt.PenJoinStyle.RoundJoin)
         self.painter.setPen(PEN_GRAY)
         self.painter.setOpacity(opacity)
 
@@ -977,8 +978,8 @@ class Renderer(QFrame):
 
         _pen = self.painter.pen()
         _pen.setWidth(5)
-        _pen.setCapStyle(Qt.RoundCap)
-        _pen.setJoinStyle(Qt.RoundJoin)
+        _pen.setCapStyle(Qt.PenCapStyle.RoundCap)
+        _pen.setJoinStyle(Qt.PenJoinStyle.RoundJoin)
         _pen.setColor(QColor(c.red(), c.green(), c.blue(), 25))
         self.painter.setPen(_pen)
 
@@ -997,7 +998,7 @@ class Renderer(QFrame):
     def draw_loading_screen(self):
         x = self.width() / 2 - 75
         y = self.height() / 2 - 10
-        self.painter.drawText(x, y, "Calculating Sliders, please wait...")
+        self.painter.drawText(int(x), int(y), "Calculating Sliders, please wait...")
         progress = int((self.sliders_current / self.num_sliders) * 100)
         self.draw_progressbar(progress)
 
@@ -1093,16 +1094,15 @@ class Renderer(QFrame):
             qrect = rect.toQRect()
             if qrect.contains(event.pos()):
                 any_inside = True
-                self.setCursor(QCursor(Qt.PointingHandCursor))
+                self.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
         if not any_inside:
-            self.setCursor(QCursor(Qt.ArrowCursor))
+            self.setCursor(QCursor(Qt.CursorShape.ArrowCursor))
         return super().mouseMoveEvent(event)
 
     def mousePressEvent(self, event):
-        for rect in self.player_info_positions:
+        for rect, player in self.player_info_positions.items():
             qrect = rect.toQRect()
             if qrect.contains(event.pos()):
-                player = self.player_info_positions[rect]
                 # toggle its membership in disabled_players, so users can click
                 # a second time to re-enable a player
                 if player in self.disabled_players:
@@ -1154,6 +1154,31 @@ class Renderer(QFrame):
         r = self.hitcircle_radius
         return math.sqrt((((x2 - x1) ** 2) + (y2 - y1) ** 2)) - r
 
+    def calculate_beatmap_stats(self, use_hr, use_ez):
+        ar = self.beatmap.ar(hard_rock=use_hr, easy=use_ez)
+        od = self.beatmap.od(hard_rock=use_hr, easy=use_ez)
+        cs = self.beatmap.cs(hard_rock=use_hr, easy=use_ez)
+
+        # see https://osu.ppy.sh/help/wiki/Beatmapping/Approach_rate
+        if ar <= 5:
+            self.preempt = 1200 + 600 * (5 - ar) / 5
+            self.fade_in = 800 + 400 * (5 - ar) / 5
+        else:
+            self.preempt = 1200 - 750 * (ar - 5) / 5
+            self.fade_in = 800 - 500 * (ar - 5) / 5
+
+        (hitwindow_50, hitwindow_100, hitwindow_300) = hitwindows(od)
+        self.hitwindow_50 = hitwindow_50
+        self.hitwindow_100 = hitwindow_100
+        self.hitwindow_300 = hitwindow_300
+
+        # how much to scale our error bar by from its 'standard' size, where
+        # each ms of error is a pixel.
+        self.error_bar_width_factor = ERROR_BAR_WIDTH / (hitwindow_50 * 2)
+
+        self.hitcircle_radius = hitradius(cs)
+
+
     def raw_view_changed(self, new_state):
         self.raw_view = new_state
         # redraw everything for the new raw view
@@ -1186,8 +1211,7 @@ class Renderer(QFrame):
             return
         use_hr = new_value == "HR"
         use_ez = new_value == "EZ"
-        cs = self.beatmap.cs(hard_rock=use_hr, easy=use_ez)
-        self.hitcircle_radius = hitradius(cs)
+        self.calculate_beatmap_stats(use_hr, use_ez)
         self.update()
 
 
